﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;

namespace MCNGen {
    /// <summary>
    /// TIEA212 Graafisten käyttöliittymien ohjelmointi. 
    /// 
    /// A simple program for generating names using markov chains.
    /// 
    /// Author: Joonas Rajala
    /// </summary>
    public partial class Nimigen : Window {

        int Order = 4;
        int MinLength = 4;
        int Amount = 40;

        public String InputText {
            get { return (String)GetValue(InputTextProperty); }
            set { SetValue(InputTextProperty, value); }
        }

        public static readonly DependencyProperty InputTextProperty =
            DependencyProperty.Register("InputText", typeof(String), typeof(TextBox), new UIPropertyMetadata(""));


        public Nimigen() {
            InitializeComponent();
        }

        /// <summary>
        /// Reads a file into a list
        /// </summary>
        /// <param name="p">path to file</param>
        /// <returns>contents of the file</returns>
        private List<string> ReadFile(string p) {

            List<string> temp = new List<string>();
            if (File.Exists(p)) {
                using (StreamReader sr = File.OpenText(p)) {
                    string s = "";
                    while ((s = sr.ReadLine()) != null) {
                        temp.Add(s);
                    }
                }
            }
            temp.Sort();
            return temp;
        }

        private List<String> createList(String s) {
            string[] asd = s.Split(new string[] { "\r\n", "\n" }, StringSplitOptions.RemoveEmptyEntries);

            return new List<string>(asd);
        }

        /// <summary>
        /// Runs the name generator
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGenerate_Click(object sender, RoutedEventArgs e) {

            List<string> source = createList(textBox1.Text);

            if (source == null || source.Count < 10) {
                MessageBox.Show(this, "Minimum input: 10 rows of at least 3 characters.");
                return;
            }
            NameGenerator ng = new NameGenerator();

            List<string> output = ng.Run(source, Order, MinLength, Amount);

            if (output == null) {
                MessageBox.Show(this, "Could not generate results. Try longer words, more variety and/or more length.");
                return;
            }

            textBox2.Text = String.Join("\n", output.ToArray());

        }

        /// <summary>
        /// Shows an error dialog regarding minimum sample input.
        /// </summary>
        private void showSampleError() {            
           
        }

        private void ButtonOpen_Click(object sender, RoutedEventArgs e) {

            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = "Source";
            dlg.DefaultExt = ".txt";
            dlg.Filter = "Text documents (.txt)|*.txt";

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true) {
                List<string> source = ReadFile(dlg.FileName);
                textBox1.Text = String.Join("\n", source.ToArray());
            }
        }

        /// <summary>
        /// Opens a save dialog for the output list.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSave_Click(object sender, RoutedEventArgs e) {

            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.FileName = "Generoidut";
            dlg.DefaultExt = ".txt";
            dlg.Filter = "Text documents (.txt)|*.txt";

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true) {
                using (StreamWriter sw = new StreamWriter(dlg.FileName)) {
                    sw.Write(textBox2.Text);
                }
            }
        }

        private void menuItemAbout_Click(object sender, RoutedEventArgs e) {
            string asd = "MCNGen v0.1\n" + "Joonas Rajala\n" + "27.4.2012";
            MessageBox.Show(asd, "About MCNGen");
        }

        /// <summary>
        /// Exits the program.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void menuItemExit_Click(object sender, RoutedEventArgs e) {
            Application.Current.Shutdown();
        }

        /// <summary>
        /// Shows the settings dialog for the name generator
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOptions_Click(object sender, RoutedEventArgs e) {
            OptionsDialog dlg = new OptionsDialog();

            dlg.Order = Order;
            dlg.MinLength = MinLength;
            dlg.Amount = Amount;
            dlg.Owner = this;

            dlg.ShowDialog();

            this.Order = dlg.Order;
            this.Amount = dlg.Amount;
            this.MinLength = dlg.MinLength;
        }

        /// <summary>
        /// Opens the help dialog
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemVewHelp_Click(object sender, RoutedEventArgs e) {
            HelpDialog dlg = new HelpDialog();

            dlg.ShowDialog();
        }
    }
}
