﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;

namespace MCNGen {
    /// <summary>
    /// Validation rule for the options dialog of Nimigen. Checks wether the input value is between two
    /// numbers.
    /// 
    /// Author: Joonas Rajala
    /// </summary>
    class OptionsValidationRule : ValidationRule {

        private int _min;
        private int _max;

        public int min {
            get { return _min; }
            set { _min = value; }
        }
        public int max {
            get { return _max; }
            set { _max = value; }
        }

        public override ValidationResult Validate(object value, System.Globalization.CultureInfo cultureInfo) {

            int i;

            if (!int.TryParse((string)value, out i)) {
                return new ValidationResult(false, "Not a number.");
            }

            if (i > _max || i < _min) {
                string s = string.Format("Value must be between {0} and {1}.", _min, _max);
                return new ValidationResult(false, s);
            }

            return new ValidationResult(true, null);
        }
    }
}
